@@ -1,6 +1,6 @@
-;;; multishell-list.el --- tabulated-list-mode for multishell shell buffers
+;;; multishell-list.el --- tabulated-list-mode for multishell shell buffers  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2016 Free Software Foundation, Inc. and Ken Manheimer
+;; Copyright (C) 2016-2020 Free Software Foundation, Inc. and Ken Manheimer
 
 ;; Author: Ken Manheimer <ken.manheimer@gmail.com>
 ;; Version: 1.1.5
@@ -10,7 +10,10 @@
 
 ;; See multishell.el for commentary, change log, etc.
 
+;; Load multishell to load this.
+
 (require 'tabulated-list)
+(eval-when-compile (require 'cl-lib))
 
 (defgroup multishell-list nil
   "Show a menu of all shell buffers in a buffer."
@@ -65,9 +68,9 @@ switch to the buffer but don't activate (or deactivate) it it."
     (with-current-buffer list-buffer
       (revert-buffer))))
 
-(defun multishell-list-delete (&optional arg)
+(defun multishell-list-delete (&optional _arg)
   "Remove current shell entry, and prompt for buffer-removal if present."
-  (interactive "P")
+  (interactive)
   (let* ((entry (tabulated-list-get-id))
          (name (multishell-name-from-entry entry))
          (name-bracketed (multishell-bracket name))
@@ -200,7 +203,9 @@ Provide for concluding minibuffer interaction if we're in completing mode."
               (not (string= (tabulated-list-get-id) entry)))
     (forward-line 1)))
 
-(defun multishell-collate-row-strings-as-numbers (a b)
+(define-obsolete-function-alias 'multishell-collate-row-strings-as-numbers
+  #'multishell-list--collate-row-strings-as-numbers "multishell 1.1.6")
+(defun multishell-list--collate-row-strings-as-numbers (a b)
   (let ((a (aref (cadr a) 0))
         (b (aref (cadr b) 0)))
     (> (string-to-number a) (string-to-number b))))
@@ -231,14 +236,14 @@ Provide for concluding minibuffer interaction if we're in completing mode."
 
 Initial sort is from most to least recently used:
 
-- First active shells, flagged with '+' a plus sign
-- Then, inactive shells, flagged with '.' a period
-- Then historical shells that currently have no buffer, flagged with 'x' an ex
+- First active shells, flagged with `+' a plus sign
+- Then, inactive shells, flagged with `.' a period
+- Then historical shells that currently have no buffer, flagged with `x' an ex
 
 \\{multishell-list-mode-map\}"
   (setq tabulated-list-format
         [;; (name width sort '(:right-align nil :pad-right nil))
-         ("#" 0 multishell-collate-row-strings-as-numbers :pad-right 1)
+         ("#" 0 multishell-list--collate-row-strings-as-numbers :pad-right 1)
          ("! " 1 t :pad-right 1)
          ("Name" 15 t)
          ("Hops" 30 t)
@@ -253,15 +258,14 @@ Initial sort is from most to least recently used:
 For duplicates, we prefer the ones that have paths."
   (let ((tally (make-hash-table :test #'equal))
         got name name-order-reversed already)
-    (mapcar #'(lambda (entry)
-                (setq name (multishell-name-from-entry entry)
-                      already (gethash name tally nil))
-                (when (not already)
-                  (push name name-order-reversed))
-                (when (or (not already) (< (length already) (length entry)))
-                  ;; Add new or replace shorter prior entry for name:
-                  (puthash name entry tally)))
-            entries)
+    (dolist (entry entries)
+      (setq name (multishell-name-from-entry entry)
+            already (gethash name tally nil))
+      (when (not already)
+        (push name name-order-reversed))
+      (when (or (not already) (< (length already) (length entry)))
+        ;; Add new or replace shorter prior entry for name:
+        (puthash name entry tally)))
     (dolist (name name-order-reversed)
       (push (gethash name tally) got))
     got))
@@ -271,7 +275,7 @@ For duplicates, we prefer the ones that have paths."
   "Edit your current and historic list of shell buffers.
 
 If optional COMPLETING is nil, we present the full
-`multishell-history' list in a popped buffer named '*Shells*'.
+`multishell-history' list in a popped buffer named `*Shells*'.
 
 In the buffer, hit ? or h for a list of commands.
 
@@ -294,11 +298,11 @@ You can get to the shells listing by recursively invoking
         (set-buffer buffer)
       (pop-to-buffer buffer))
     (multishell-list-mode)
-    (progv
+    (cl-progv
         ;; Temporarily assign multishell-history only when completing:
         (when completing '(multishell-history))
         (when completing
-          (list (multishell-list-cull-dups (mapcar 'substring-no-properties
+          (list (multishell-list-cull-dups (mapcar #'substring-no-properties
                                                    completing))))
       (tabulated-list-print))
     (when completing
@@ -307,6 +311,5 @@ You can get to the shells listing by recursively invoking
       (multishell-list-goto-item-by-entry from-entry))))
 
 (provide 'multishell-list)
-(require 'multishell)
 
 ;;; multishell-list.el ends here
