@@ -1,11 +1,12 @@
-;;; multishell.el --- Easily use multiple shell buffers, local and remote.
+;;; multishell.el --- Easily use multiple shell buffers, local and remote  -*- lexical-binding:t -*-
 
-;; Copyright (C) 1999-2016 Free Software Foundation, Inc.
+;; Copyright (C) 1999-2020 Free Software Foundation, Inc. and Ken Manheimer
 
 ;; Author: Ken Manheimer <ken.manheimer@gmail.com>
-;; Version: 1.1.5
+;; Version: 1.1.7
 ;; Created: 1999 -- first public availability
 ;; Keywords: processes
+;; Package-Requires: ((cl-lib "0.5"))
 ;; URL: https://github.com/kenmanheimer/EmacsMultishell
 ;;
 ;;; Commentary:
@@ -46,7 +47,7 @@
 ;;
 ;; * Manage your list of shells, current and past, as a collection.
 ;;
-;; See the `multishell-pop-to-shell` docstring for details.
+;; See the `multishell-pop-to-shell' docstring for details.
 ;;
 ;; Customize-group `multishell' to select and activate a keybinding and set
 ;; various behaviors. Customize-group `savehist' to preserve buffer
@@ -59,6 +60,13 @@
 ;;
 ;; Change Log:
 ;;
+;; * 2020-10-28 1.1.7 Ken Manheimer:
+;;   - Forward compatibility: 'cl-progv' rather than 'progv', resolves
+;;     multishell-list error in recent emacs versions.
+;;   - Incorporate gnu refinements (thanks!)
+;; * 2016-06-27 1.1.6 Ken Manheimer (unreleased):
+;;   - When starting a remote shell, if cd fails to an inital remote
+;;     directory, try again without the cd.
 ;; * 2016-02-11 1.1.5 Ken Manheimer:
 ;;   - Rectify multishell list sorting to preserve recentness
 ;;   - Increment the actual multishell-version setting, neglected for 1.1.4.
@@ -159,7 +167,7 @@
 (require 'savehist)
 (require 'multishell-list)
 
-(defvar multishell-version "1.1.5")
+(defvar multishell-version "1.1.7")
 (defun multishell-version (&optional here)
   "Return string describing the loaded multishell version."
   (interactive "P")
@@ -179,8 +187,8 @@ with allout-mode."
 (defcustom multishell-command-key "\M- "
   "The key to use if `multishell-activate-command-key' is true.
 
-You can instead manually bind `multishell-pop-to-shell` using emacs
-lisp, eg: (global-set-key \"\\M- \" 'multishell-pop-to-shell)."
+You can instead manually bind `multishell-pop-to-shell' using emacs
+lisp, eg: (global-set-key \"\\M- \" \\='multishell-pop-to-shell)."
   :type 'key-sequence)
 
 (defvar multishell--responsible-for-command-key nil
@@ -197,28 +205,25 @@ If optional UNBIND is true, globally unbind the key.
 
 * `multishell-activate-command-key' - Set this to get the binding or not.
 * `multishell-command-key' - The key to use for the binding, if appropriate."
-  (cond (unbind
-         (when (and (boundp 'multishell-command-key) multishell-command-key)
-           (global-unset-key multishell-command-key)))
-        ((not (and (boundp 'multishell-activate-command-key)
-                   (boundp 'multishell-command-key)))
-         nil)
-        ((and multishell-activate-command-key multishell-command-key)
-         (setq multishell--responsible-for-command-key t)
-         (global-set-key multishell-command-key 'multishell-pop-to-shell))))
+  (when (bound-and-true-p multishell-command-key)
+    (if unbind
+        (global-unset-key multishell-command-key)
+      (when (bound-and-true-p multishell-activate-command-key)
+        (setq multishell--responsible-for-command-key t)
+        (global-set-key multishell-command-key 'multishell-pop-to-shell)))))
 
 (defcustom multishell-activate-command-key nil
   "Set this to impose the `multishell-command-key' binding.
 
-You can instead manually bind `multishell-pop-to-shell` using emacs
-lisp, eg: (global-set-key \"\\M- \" 'multishell-pop-to-shell)."
+You can instead manually bind `multishell-pop-to-shell' using emacs
+lisp, eg: (global-set-key \"\\M- \" \\='multishell-pop-to-shell)."
   :type 'boolean
-  :set 'multishell-activate-command-key-setter)
+  :set #'multishell-activate-command-key-setter)
 
 ;; Implement the key customization whenever the package is loaded:
 (if (fboundp 'with-eval-after-load)
     (with-eval-after-load "multishell"
-			  (multishell-implement-command-key-choice))
+      (multishell-implement-command-key-choice))
   (eval-after-load "multishell"
     '(multishell-implement-command-key-choice)))
 
@@ -240,8 +245,7 @@ When set, the history entry for shells started with explicit
 paths will track the shell's current working directory.
 
 If `savehist-save-minibuffer-history' is enabled, the current
-working directory of shells will be conveyed between emacs
-sessions."
+working directory of shells will be conveyed between Emacs sessions."
  :type 'boolean)
 
 (defvar multishell-history nil
@@ -300,6 +304,7 @@ Returns non-nil iff any changes were made."
             did-revisions t))
     did-revisions))
 
+;;;###autoload
 (defun multishell-history-entries (name)
   "Return `multishell-history' entry that starts with NAME, or nil if none."
   (let (got)
@@ -323,29 +328,29 @@ historical shells, collectively, using `multishell-list' - see below.
 Customize-group `multishell' to set up a key binding and tweak behaviors.
 
 Manage your collection of current and historical shells by
-recursively invoking \\[multishell-pop-to-shell] at the
-`multishell-pop-to-shell' universal argument prompts, eg:
+recursively invoking \\[multishell-pop-to-shell] at the `multishell-pop-to-shell'
+universal argument prompts, eg:
 
   \\[universal-argument] \\[multishell-pop-to-shell] \\[multishell-pop-to-shell]
 
-\(That will be just a few keys if you do the above
-customization.) Hit ? in the listing buffer for editing commands.
+\(That will be just a few keys if you do the above customization.)
+
+Hit ? in the listing buffer for editing commands.
 
 ==== Basic operation:
 
- - If the current buffer is shell-mode (or shell-mode derived)
-   buffer then focus is moved to the process input point.
+ - If the current buffer is in shell-mode then focus is moved to
+   the process input point.
 
-   \(You can use a universal argument go to a different shell
-   buffer when already in a buffer that has a process - see
-   below.)
+   \(Use a universal argument go to a different shell buffer
+   when already in a buffer that has a process - see below.)
 
- - If not in a shell buffer (or with universal argument), go to a
-   window that is already showing the (a) shell buffer, if any.
+ - If not in a shell buffer, go to a window that is already
+   showing a shell buffer, if any.
 
-   In this case, the cursor is left in its prior position in the
-   shell buffer. Repeating the command will then go to the
-   process input point, per the first item in this list.
+   In this case, the cursor is not moved to the process input
+   point. Repeating the command once you're in the buffer will
+   then move the cursor to the process input point.
 
    We respect `pop-up-windows', so you can adjust it to set the
    other-buffer/same-buffer behavior.
@@ -358,20 +363,20 @@ was disconnected or otherwise stopped, it's resumed.
 
 ===== Universal arg to start and select between named shell buffers:
 
-You can name alternate shell buffers to create or return to, by
-prefixing your \\[multishell-pop-to-shell] invocation with single or double
+You can assign a distinct name to new shell buffers by prefixing
+your \\[multishell-pop-to-shell] invocation with a single or double
 `universal-argument', \\[universal-argument]:
 
  - With a single universal argument, prompt for the buffer name
    to use (without the asterisks that shell mode will put around
-   the name), defaulting to 'shell'.
+   the name), defaulting to `shell'.
 
    Completion is available.
 
    This combination makes it easy to start and switch across
    multiple shell restarts.
 
- - A double universal argument will prompt for the name *and* set
+ - A double universal argument will prompt for the name and set
    the default to that name, so the target shell becomes the
    primary.
 
@@ -379,31 +384,31 @@ prefixing your \\[multishell-pop-to-shell] invocation with single or double
    setting across emacs restarts.
 
  - Manage your collection of current and historical shells by
-   recursively invoking \\[multishell-pop-to-shell] at either of the
-   `multishell-pop-to-shell' universal argument prompts, or at any
-   time via \\[multishell-list]. Hit ? in the listing buffer for
-   editing commands.
+   recursively invoking \\[multishell-pop-to-shell] at the `multishell-pop-to-shell'
+   universal argument prompts, or at any time via
+   \\[multishell-list]. Hit ? in the listing buffer for editing
+   commands.
 
 ===== Select starting directory and remote host:
 
 The shell buffer name you give to the prompt for a universal arg
 can include an appended path. That will be used for the startup
 directory. You can use tramp remote syntax to specify a remote
-shell. If there is an element after a final '/', that's used for
+shell. If there is an element after a final `/', that's used for
 the buffer name. Otherwise, the host, domain, or path is used.
 
 For example:
 
-* '#root/sudo:root@localhost:/etc' for a buffer named \"*#root*\" with a
+* `#root/sudo:root@localhost:/etc' for a buffer named \"*#root*\" with a
   root shell starting in /etc.
 
-* '/ssh:example.net:' for a shell buffer in your homedir on example.net. 
+* `/ssh:example.net:' for a shell buffer in your homedir on example.net.
   The buffer will be named \"*example.net*\".
 
-* '#ex/ssh:example.net|sudo:root@example.net:/var/log' for a root shell
+* `#ex/ssh:example.net|sudo:root@example.net:/var/log' for a root shell
   starting in /var/log on example.net named \"*#ex*\".
 
-* 'interior/ssh:gateway.corp.com|ssh:interior.corp.com:' to go
+* `interior/ssh:gateway.corp.com|ssh:interior.corp.com:' to go
   via gateway.corp.com to your homedir on interior.corp.com.  The
   buffer will be named \"*interior*\". You could append a sudo
   hop to the path, combining the previous example, and so on.
@@ -480,7 +485,7 @@ customize the savehist group to activate savehist."
 
     ;; Situate:
 
-    (cond 
+    (cond
 
      ((and (or curr-buff-proc from-buffer-is-shell)
            (not arg)
@@ -569,7 +574,7 @@ Return the last entry deleted."
     (error
      (message "multishell-kill-buffer-query-function error: %s" err)))
   t)
-(add-hook 'kill-buffer-query-functions 'multishell-kill-buffer-query-function)
+(add-hook 'kill-buffer-query-functions #'multishell-kill-buffer-query-function)
 
 (defun multishell-get-visible-window-for-buffer (buffer)
   "Return visible window containing buffer."
@@ -652,7 +657,7 @@ Return what's provided, if anything, else nil."
 (defun multishell-resolve-target-name-and-path (shell-spec)
   "Given name/tramp-style address shell spec, resolve buffer name and directory.
 
-The name is the part of the string up to the first '/' slash, if
+The name is the part of the string up to the first `/' slash, if
 any. Missing pieces are filled in from remote path elements, if
 any, and multishell history. Given a tramp-style remote address
 and no name part, either the user@host is used for the buffer
@@ -693,15 +698,22 @@ and path nil if none is resolved."
 (declare-function tramp-dissect-file-name "tramp")
 (declare-function tramp-cleanup-connection "tramp")
 
-(defun multishell-start-shell-in-buffer (path)
-  "Start, restart, or continue a shell in BUFFER-NAME on PATH."
+(defun multishell-start-shell-in-buffer (where)
+  "Start, restart, or continue a shell in current-buffer on WHERE.
+
+If WHERE is remote and includes a directory, cd to that directory on the
+remote host.
+
+If cd fails to an included remote directory, try again without the cd."
   (let* ((is-active (comint-check-proc (current-buffer))))
 
-    (when (and path (not is-active))
+    (when (and where (not is-active))
 
-      (when (and (derived-mode-p 'shell-mode) (file-remote-p path))
+      ;; FIXME: file-remote-p does not imply Tramp.
+      ;; Why do we need to do something special for Tramp here?
+      (when (and (derived-mode-p 'shell-mode) (file-remote-p where))
         ;; Returning to disconnected remote shell - do some tidying.
-        ;; Without this cleanup, occasionally restarting a disconnected
+        ;; FIXME: Without this cleanup, occasionally restarting a disconnected
         ;; remote session, particularly one that includes sudo, results in
         ;; an untraceable "Args out of range" error. That never happens if
         ;; we precedeed connection attempts with this cleanup -
@@ -710,8 +722,25 @@ and path nil if none is resolved."
          (tramp-dissect-file-name default-directory 'noexpand)
          'keep-debug 'keep-password))
 
-      (when (file-remote-p path) (message "Connecting to %s" path))
-      (cd path))
+      (if (not (file-remote-p where))
+          (cd where)
+        (message "Connecting to %s" where)
+        (condition-case err
+            (cd where)
+          ;; "cd: No such directory found via CDPATH environment variable"
+          (error
+           (if (string=
+                (cadr err)
+                "No such directory found via CDPATH environment variable")
+               ;; Try again without dir part of remote where:
+               (let* ((final-colon-at (string-match ":[^:]+$" where))
+                      (sans-dir-path (substring where 0 (1+ final-colon-at)))
+                      (dir-path (substring where (1+ final-colon-at))))
+                 (message "Failed to cd to %s, trying again without..."
+                          dir-path)
+                 (sit-for .5)
+                 (cd sans-dir-path))
+             (signal (car err) (cdr err)))))))
 
     (shell (current-buffer))))
 
@@ -736,7 +765,6 @@ and path nil if none is resolved."
   (let* ((entries (multishell-history-entries name)))
     (dolist (entry entries)
       (let* ((name-path (multishell-split-entry entry))
-             (name (car name-path))
              (path (or (cadr name-path) "")))
         (when path
           (let* ((old-localname (or (file-remote-p path 'localname)
@@ -799,6 +827,7 @@ Returns nil for empty parts, rather than the empty string."
   (if (not (string= (substring name -1) "*"))
       (setq name (concat name "*")))
   name)
+;;;###autoload
 (defun multishell-unbracket (name)
   "Return a copy of name, removing asterisks, if any, at beginning and end."
   (if (string= (substring name 0 1) "*")
